from __future__ import annotations
from collections import defaultdict
from os.path import splitext
from pathlib import Path

import pendulum

from .minioserver import MinioServer
from capsule import settings


# --------------------------------------------------------------------------------
class BackupFile:
    """
    Dump a database or compress a directory to file and backup to remote server.

    Create daily / weekly / monthly backup files, and
    maintain designated archive of copies on the remote server
    """
    base_prefix = ''
    file_path = ''

    # Scrub old files after after file limit exceeded
    scrub_local_files = True
    scrub_remote_files = True

    def __init__(self, ctx, filename: str, interval: str = ''):
        self.ctx = ctx
        self.base_filename, self.extension = splitext(filename)
        self.date = pendulum.now()
        self._filename = ''
        self._interval = interval
        self.server = MinioServer(
            prefix=f'{self.base_prefix}/{self.interval()}',
            filename=self.filename(),
        )

    def process(self):
        """
        Perform all file preparations and upload to Minio server.
        Override on subclasses to add additional processing.
        """
        self.upload()
        self.scrub()

    def interval(self) -> str:
        """
        Calculate whether this is a daily / weekly / monthly backup file.
        """
        if not self._interval:
            yesterday = self.date.subtract(days=1)

            if yesterday.month != self.date.month:
                self._interval = 'monthly'
            elif yesterday.week_of_year != self.date.week_of_year:
                self._interval = 'weekly'
            else:
                self._interval = 'daily'

        return self._interval

    def max_age(self) -> tuple[int, pendulum]:
        """
        Return oldest date of any backup files that should be retained.
        All files older than this date should be deleted.
        """
        interval = self.interval()
        DAILY_MAX_FILE_COUNT = settings.DAILY_MAX_FILE_COUNT
        MONTHLY_MAX_FILE_COUNT = settings.MONTHLY_MAX_FILE_COUNT
        WEEKLY_MAX_FILE_COUNT = settings.WEEKLY_MAX_FILE_COUNT

        if interval == 'monthly':
            return MONTHLY_MAX_FILE_COUNT, self.date.subtract(months=MONTHLY_MAX_FILE_COUNT)

        if interval == 'weekly':
            return WEEKLY_MAX_FILE_COUNT, self.date.subtract(weeks=WEEKLY_MAX_FILE_COUNT)

        return DAILY_MAX_FILE_COUNT, self.date.subtract(days=DAILY_MAX_FILE_COUNT)

    def filename(self) -> str:
        """
        Calculate backup file name from date, interval type and base file name.
        """
        if not self._filename:
            bdate = self.date.strftime('%Y%m%d')
            self._filename = f'{self.file_path}/{self.base_filename}-{bdate}{self.extension}'
        return self._filename

    def upload(self):
        """
        Upload current file to remote server.
        """
        self.server.upload()

    def scrub(self):
        """
        Scrub remote / local backup files after age limit is exceeded.
        """
        max_file_count, max_age = self.max_age()
        obsolete_ts = int(max_age.timestamp())

        if self.scrub_local_files:
            pattern = f'{self.base_filename}-*{self.extension}'
            backup_files = list(Path(self.file_path).glob(pattern))

            # Archive interval not considered for local backup files.
            if len(backup_files) > settings.DAILY_MAX_FILE_COUNT:
                for backup_file in backup_files:
                    if backup_file.stat().st_mtime < obsolete_ts:
                        backup_file.unlink(missing_ok=True)

        if self.scrub_remote_files:
            self.server.scrub(max_file_count, max_age)

    def backups(self, name: str = '', interval: str = '') -> dict[str, list[str]]:
        """
        Get a dictionary of all backup files in self.base_prefix, grouped by interval.
        """
        all_backups = defaultdict(list)
        prefix = self.base_prefix if not interval else f'{self.base_prefix}/{interval}'
        backups = self.server.list_objects(prefix=prefix)

        for backup in backups:
            backup_file = backup.object_name.split('/')
            backup_interval = backup_file[1]
            db_name = backup_file[-1].split('-')[0]
            if not name or db_name == name:
                all_backups[backup_interval].append(backup_file[-1])

        return all_backups


__all__ = [
    'BackupFile',
]
