from __future__ import annotations
import logging
import os
import pwd

import psycopg2
from .base import BackupFile
from ..settings import OPT_DIR

logger = logging.getLogger(__name__)


# --------------------------------------------------------------------------------
def get_databases(excluded: list[str] = ()) -> list[str]:
    """
    Get list of databases created on the system.

    ['logcabin', 'tabula', 'odoo']
    """
    excluded_dbs = {'excluded': excluded or ('template0', 'template1', 'postgres')}
    sql = """
    SELECT datname FROM pg_database 
    WHERE datname NOT IN %(excluded)s;
    """
    with psycopg2.connect('user=postgres') as conn:
        with conn.cursor() as cursor:
            cursor.execute(sql % excluded_dbs)
            return [db[0] for db in cursor.fetchall()]


# --------------------------------------------------------------------------------
# Exclude tables from databases when creating backup dump files
DATABASE_EXCLUDE_TABLES = {
    'logcabin': (
        "django_session",
        "console_sessions_session",
        "overrides_autofixes",
        "overrides_overblocks",
        "reporter_logline*",
        "reporter_summaryantivirus*",
        "reporter_summaryapplication*",
        "reporter_summaryclassifier*",
        "reporter_summarydomains*",
        "reporter_summarydomainsblocked*",
        "reporter_summaryhits*",
        "reporter_summarymediaviews*",
        "reporter_summarypagetitles*",
        "reporter_summarysearches*",
        "mailer_message",
        "mailer_messagelog",
    ),
}


# --------------------------------------------------------------------------------
class BackupDatabase(BackupFile):
    """
    Dump Database to file and backup to remote server.

    Create daily / weekly / monthly database files, and
    maintain designated archive of copies on the remote server
    """
    file_path = f'{OPT_DIR}/pgbackups'
    base_prefix = 'databases'

    def process(self):
        """
        Perform all database dump, upload and cleanup operations.
        """
        self.dump()
        super().process()

    def dump(self):
        """
        Dump database to local file.
        """
        if not os.path.exists(self.file_path):
            os.makedirs(self.file_path, exist_ok=True)

        database_name = self.base_filename
        base_cmd = f'pg_dump -U postgres -F c {database_name}'

        exclude_tables = DATABASE_EXCLUDE_TABLES.get(database_name, [])
        if exclude_tables:
            tables = " ".join([f'--exclude-table-data "{table}"' for table in exclude_tables])
            base_cmd = f'{base_cmd} {tables}'

        cmd = f'{base_cmd} -f {self.filename()}'
        self.ctx.run(cmd, pty=True)

        # set owner to nginx if that user exists on the system, so the
        # directory can be served via nginx web server for list viewing
        try:
            pwd.getpwnam('nginx')
            self.ctx.run(f'chown -R nginx:nginx {self.file_path}', pty=True)
        except KeyError:
            pass

    def db_list(self, interval: str) -> list[str]:
        """
        Get list of all database backups on the remote server.
        """
        names = set()
        prefix = self.base_prefix if not interval else f'{self.base_prefix}/{interval}'
        backups = self.server.list_objects(prefix=prefix)

        for backup in backups:
            backup_name = backup.object_name.split('/')[-1].split('-')[0]
            names.add(backup_name)

        return sorted(list(names))


__all__ = (
    'BackupDatabase',
    'get_databases',
)
