import os
import tarfile
from pathlib import Path

from invoke import task

from ..backups import BackupEtc, MinioServer


@task
def backup(ctx):
    """
    Backup all contents of the /etc directory.
    """
    print('Backing up /etc... ', end='', flush=True)
    BackupEtc(ctx, filename='etc.tbz2').process()
    print('Done')


@task(
    help={
        'interval': 'File archive interval - (daily, weekly, monthly)',
    },
)
def backup_files(ctx, interval=''):
    """
    Show all Etc backup files for specified interval.
    """
    backups = BackupEtc(ctx, filename='', interval='').backups(interval=interval)

    for interval, db_backups in backups.items():
        print(f'\n{interval.title()} backup files')
        for bf in db_backups:
            print(bf)
        print()


@task(
    help={
        'interval': 'File archive interval - (daily, weekly, monthly)',
        'filename': 'Remote backup filename to download',
        'destination': 'Directory to download to',
    },
)
def download(ctx, interval, filename, destination):
    """
    Download specified Etc backup tarball from the backup server.
    """
    if destination == '/':
        print('Never download backup to the root directory!')
        print('Un-tarring it there will overwrite the entire contents of /etc!')
        return
    destination = destination.rstrip('/')

    dst = f"{destination}/{filename}"
    MinioServer(filename, prefix=f'etc/{interval}').download(dst)

    if not Path(dst).exists():
        print(f'Unable to download {filename}.')
        return

    with tarfile.open(dst) as tarball:
        tarball.extractall(path=f'{destination}/etcbackup')

    os.unlink(dst)

    print(f'Downloaded and extracted {filename} into {destination}/etcbackup')
    print('Browse and select files to manually restore.')


__all__ = (
    'backup',
    'backup_files',
    'download',
)
